
/* * * * * * * * * * * * * Author's note * * * * * * * * * * * *\
*   _       _   _       _   _       _   _       _     _ _ _ _   *
*  |_|     |_| |_|     |_| |_|_   _|_| |_|     |_|  _|_|_|_|_|  *
*  |_|_ _ _|_| |_|     |_| |_|_|_|_|_| |_|     |_| |_|_ _ _     *
*  |_|_|_|_|_| |_|     |_| |_| |_| |_| |_|     |_|   |_|_|_|_   *
*  |_|     |_| |_|_ _ _|_| |_|     |_| |_|_ _ _|_|  _ _ _ _|_|  *
*  |_|     |_|   |_|_|_|   |_|     |_|   |_|_|_|   |_|_|_|_|    *
*                                                               *
*                     http://www.humus.name                     *
*                                                                *
* This file is a part of the work done by Humus. You are free to   *
* use the code in any way you like, modified, unmodified or copied   *
* into your own work. However, I expect you to respect these points:  *
*  - If you use this file and its contents unmodified, or use a major *
*    part of this file, please credit the author and leave this note. *
*  - For use in anything commercial, please request my approval.     *
*  - Share your work and ideas too as much as you can.             *
*                                                                *
\* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

#include "App.h"

BaseApp *app = new App();

void App::resetCamera(){
	camPos = vec3(7.5f, 1, -12);
	wx = 0.1f;
	wy = 0.0f;
}

bool App::init(){
	sphere = new Model();
	sphere->createSphere(0);

	speed = 10.0f;

	return true;
}

void App::exit(){
	delete sphere;
}

bool App::load(){
	// Shaders
	if ((shader = renderer->addShader("shader.shd")) == SHADER_NONE) return false;

	// Rasterizerstates
	if ((wireFrame = renderer->addRasterizerState(CULL_BACK, WIREFRAME)) == RS_NONE) return false;

	// Upload map to vertex/index buffer
	if (!sphere->makeDrawable(renderer, true, shader)) return false;

	return true;
}

void App::unload(){

}

void App::drawSpheres(const float4x4 &viewProj, const float3 &color, const bool wireframe){
	renderer->reset();
	renderer->setShader(shader);
	renderer->setRasterizerState(wireframe? wireFrame : cullBack);
	renderer->apply();

	renderer->setShaderConstant4x4f("viewProj", viewProj);
	renderer->setShaderConstant3f("color", color);
	for (int i = 0; i < 4; i++)
	{
		renderer->setShaderConstant3f("pos", float3(i * 5.0f, 0, 0));
		renderer->setShaderConstant1f("radius", 2.0f);
		renderer->setShaderConstant1i("tessellation", i);
		renderer->applyConstants();

		sphere->draw(renderer);
	}
}

void App::drawFrame(){
	const float near_plane = 0.2f;
	const float far_plane = 40.0f;

	float4x4 projection = toD3DProjection(perspectiveMatrixY(1.2f, width, height, near_plane, far_plane));
	float4x4 view = rotateXY(-wx, -wy);
	view.translate(-camPos);
	float4x4 viewProj = projection * view;
	float4x4 viewProjBias = translate(0, 0, -0.0001f) * viewProj;

	renderer->clear(true, true, false, float4(0, 0, 0, 0), 1.0f, 0);

	drawSpheres(viewProj,     float3(0.1f, 0.2f, 0.15f), false);
	drawSpheres(viewProjBias, float3(0.0f, 1.0f, 0.00f), true);
}
